/*
 * Copyright (c) 1998-2008 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 *
 * This file contains Original Code and/or Modifications of Original Code
 * as defined in and that are subject to the Apple Public Source License
 * Version 2.0 (the 'License'). You may not use this file except in
 * compliance with the License. Please obtain a copy of the License at
 * http://www.opensource.apple.com/apsl/ and read it before using this
 * file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
 * Please see the License for the specific language governing rights and
 * limitations under the License.
 *
 * @APPLE_LICENSE_HEADER_END@
 */

/************************************************************************/
/**
 * PDJ USB Audio Driver for Mac OS X
 *
 * @file	PdjUsbAudioClip.h
 *
 * @brief	整数型←→浮動小数点型の変換．
 *			デバイスは整数型データを扱い、Mac OS X の Audio HAL は浮動小数点型データを扱う．
 *			このファイルはＣ言語で書かれており、ライブラリ（libDJM-V10FloatLib.a）
 *			としてコンパイルされたのち、kext にリンクされる．
 *
 * This code is based on AppleUSBAudio-273.4.1 for Mac OS 10.6.4.
 */
/************************************************************************/
#ifndef CC_PDJ_USB_AUDIO_CLIP_H_FILE_
#define CC_PDJ_USB_AUDIO_CLIP_H_FILE_

extern "C" {

/*------------------------------------------*/
/*				型定義						*/
/*------------------------------------------*/

// floating point types
typedef float		Float32;
typedef double		Float64;

/*------------------------------------------*/
/*				外部関数					*/
/*------------------------------------------*/

IOReturn
clipPdjUsbAudioToOutputStream(
	const void					*mixBuf,
	void						*sampleBuf,
	UInt32						firstSampleFrame,
	UInt32						numSampleFrames,
	const IOAudioStreamFormat	*streamFormat
);

IOReturn
convertFromPdjUsbAudioInputStream_NoWrap(
	const void					*sampleBuf,
	void						*destBuf,
	UInt32						firstSampleFrame,
	UInt32						numSampleFrames,
	const IOAudioStreamFormat	*streamFormat
);

} // extern "C"

#endif /* CC_PDJ_USB_AUDIO_CLIP_H_FILE_ */
