/*
 * Copyright (c) 1998-2008 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 *
 * This file contains Original Code and/or Modifications of Original Code
 * as defined in and that are subject to the Apple Public Source License
 * Version 2.0 (the 'License'). You may not use this file except in
 * compliance with the License. Please obtain a copy of the License at
 * http://www.opensource.apple.com/apsl/ and read it before using this
 * file.
 *
 * The Original Code and all software distributed under the License are
 * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
 * Please see the License for the specific language governing rights and
 * limitations under the License.
 *
 * @APPLE_LICENSE_HEADER_END@
 */

/************************************************************************/
/**
 * PDJ USB Audio Driver for Mac OS X
 *
 * @file	PdjUsbAudioPlugin.h
 *
 * @brief	IOService クラスの派生クラス．
 *			オーディオ入出力データに対し追加の処理を行うプラグイン．
 *			PdjUsbAudioStream または PdjUsbAudioEngine に
 *			アタッチすると有効になる．
 *			アタッチするには Info.plist を書き換える必要がある．
 *
 * This code is based on AppleUSBAudio-273.4.1 for Mac OS 10.6.4.
 */
/************************************************************************/
#ifndef PDJ_USB_AUDIO_PLUGIN_H_FILE_
#define PDJ_USB_AUDIO_PLUGIN_H_FILE_

/*------------------------------------------*/
/*			PdjUsbAudioPlugin クラス		*/
/*------------------------------------------*/

#pragma mark - class PdjUsbAudioPlugin -

class PdjUsbAudioPlugin : public IOService
{
	OSDeclareDefaultStructors(PdjUsbAudioPlugin)

public:

	typedef IOService					super;

protected:

	PdjUsbAudioEngine				*mOurProvider;

	struct ExpansionData
	{
		PdjUsbAudioStream			*streamProvider;
	};

	ExpansionData					*reserved;

public:

#pragma mark - Public Methods -

	bool
	start(
		IOService					*provider
	);

	void
	stop(
		IOService					*provider
	);

	void
	pluginSetConfigurationApp(
		const char					*bundleID
	);

	IOReturn
	pluginDeviceRequest(
		StandardUSB::DeviceRequest	*request,
		IOMemoryDescriptor			*dataBuffer,
		IOUSBHostCompletion			*completion = NULL
	);

	IOReturn
	pluginInit(
		IOService					*provider,
		UInt16						vendorID,
		UInt16						productID
	);

	IOReturn
	pluginSetDirection(
		IOAudioStreamDirection		direction
	);

	IOReturn
	pluginSetFormat(
		const IOAudioStreamFormat	*const newFormat,
		const IOAudioSampleRate		*const newSampleRate
	);

	IOReturn
	pluginStart(
		void
	);

	IOReturn
	pluginStop(
		void
	);

	IOReturn
	pluginReset(
		void
	);

	IOReturn
	pluginProcessOutput(
		float						*mixBuf,
		UInt32						numSampleFrames,
		UInt32						numChannels
	);

	IOReturn
	pluginProcessInput(
		float						*destBuf,
		UInt32						numSampleFrames,
		UInt32						numChannels
	);

private:

	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin,  0);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin,  1);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin,  2);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin,  3);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin,  4);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin,  5);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin,  6);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin,  7);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin,  8);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin,  9);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin, 10);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin, 11);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin, 12);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin, 13);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin, 14);
	OSMetaClassDeclareReservedUnused(PdjUsbAudioPlugin, 15);
};

#endif /* PDJ_USB_AUDIO_PLUGIN_H_FILE_ */
